﻿using ProjectLainer.Exceptions;
using System.Numerics;

namespace ProjectLainer.Generics
{
    internal class SetGeneric<T>
    where T : class
    {
        private readonly List<T?> _places;
        public int Count => _places.Count;
        private readonly int _maxCount;
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }
        
        public void Insert(T lainer)
        {
            if (_places.Count == _maxCount)
            {
                throw new StorageOverflowException(_maxCount);
            }
            Insert(lainer, 0);
        }
       
        public void Insert(T lainer, int position) 
        {
            if (position >= 0 && position <= Count)
            {
                _places.Insert(position, lainer);
            }
            else
            {
                throw new Exception("Неверная позиция для вставки");
            }
        }

        public void Remove(int position)
        {
            if (position < Count && position >= 0)
            {
                _places.RemoveAt(position);
            }
            else
            {
                throw new LainerNotFoundException(position);
            }
        }
        public T? this[int position]
        {
            get
            {
                if(position < 0 || position >= Count)
                {
                    return null;
                }
                return _places[position];
            }
            set
            {
                if (position < 0 || position >= Count || Count >= _maxCount)
                {
                    return;
                }
                _places.Insert(position, value);
            }
        }
        public IEnumerable<T?> GetLainers(int? maxLainers = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxLainers.HasValue && i == maxLainers.Value)
                {
                    yield break;
                }
            }
        }
    }
}

